@@ -5,13 +5,15 @@ from typing import Any, Callable, Dict, Iterable, List, Optional, Set, Tuple
 
 import numpy as np
 import pandas as pd
-from pandas.core.dtypes.base import ExtensionDtype
-from triad.utils.convert import as_type
-from triad.utils.iter import EmptyAwareIterable, Slicer
-from triad.utils.json import loads_no_dup
-from triad.utils.string import validate_triad_var_name
-
 import pyarrow as pa
+from pandas.core.dtypes.base import ExtensionDtype
+
+from triad.constants import TRIAD_VAR_QUOTE
+
+from .convert import as_type
+from .iter import EmptyAwareIterable, Slicer
+from .json import loads_no_dup
+from .schema import move_to_unquoted, quote_name, unquote_name
 
 TRIAD_DEFAULT_TIMESTAMP = pa.timestamp("us")
 
@@ -97,16 +99,6 @@ _PA_TO_PANDAS_EXTENSION_TYPE_MAP: Dict[pa.DataType, ExtensionDtype] = {
 _SPECIAL_TOKENS: Set[str] = {",", "{", "}", "[", "]", "<", ">", ":"}
 
 
-def validate_column_name(expr: str) -> bool:
-    """Check if `expr` is a valid column name.
-    See :func:`~triad.utils.string.validate_triad_var_name`
-
-    :param expr: column name expression
-    :return: whether it is valid
-    """
-    return validate_triad_var_name(expr)
-
-
 def expression_to_schema(expr: str) -> pa.Schema:
     """Convert schema expression to `pyarrow.Schema`.
 
@@ -381,7 +373,8 @@ class SchemaedDataPartitioner(object):
 
 
 def _field_to_expression(field: pa.Field) -> str:
-    return f"{field.name}:{_type_to_expression(field.type)}"
+    name = quote_name(field.name)
+    return f"{name}:{_type_to_expression(field.type)}"
 
 
 def _type_to_expression(dt: pa.DataType) -> str:
@@ -427,8 +420,6 @@ def _parse_types(v: Any):
 
 def _construct_struct(obj: Dict[str, Any]) -> Iterable[pa.Field]:
     for k, v in obj.items():
-        if not validate_column_name(k):
-            raise SyntaxError(f"{k} is not a valid field name")
         yield pa.field(k, _parse_types(v))
 
 
@@ -450,7 +441,7 @@ def _parse_type(expr: str) -> pa.DataType:
 def _parse_type_function(expr: str) -> Tuple[str, List[str]]:
     p = expr.split("(", 1)
     name = p[0].strip()
-    assert validate_column_name(name), f"Invalid expression {expr}"
+    assert name.isidentifier(), f"Invalid expression {expr}"
     if len(p) == 1:
         return name, []
     arg_expr = p[1].strip().rstrip(")")
@@ -463,7 +454,14 @@ def _parse_tokens(expr: str) -> Iterable[str]:
     expr += ","
     last = 0
     skip = False
-    for i in range(len(expr)):
+    i = 0
+    while i < len(expr):
+        if expr[i] == TRIAD_VAR_QUOTE:
+            e = move_to_unquoted(expr, i)
+            s = unquote_name(expr[i:e])
+            yield '"' + s.replace("\\", "\\\\").replace('"', '\\"') + '"'
+            last = i = e
+            continue
         if expr[i] == "(":
             skip = True
         if expr[i] == ")":
@@ -479,6 +477,7 @@ def _parse_tokens(expr: str) -> Iterable[str]:
             else:
                 yield expr[i]
             last = i + 1
+        i += 1
 
 
 def _to_pynone(obj: Any) -> Any:
